#pragma NOIV	// tells compiler not to generate interrupt vectors
				// Note that we have to edit USBJmpTB if we wanna add
				// other interrupts later

/*=============================================================================
	File:      SyncBulk.c
	Contents:   Hooks required to implement USB peripheral function.
	Comments:	Original file was given by a Cypress example, this
				one is edited only so there might me reserved rights:

				Copyright (c) 2000 Cypress Semiconductor All rights reserved
=============================================================================*/

#include "fx2.h"
#include "fx2regs.h"			// includes FX2 register adressing
//#include "fx2sdly.h"            // includes SYNCDELAY macro


/*=============================================================================
	Pre-Processor defines:   	
=============================================================================*/
// nothing to do here 

/*=============================================================================
	Global variables:
	External variables which belong to the Anchor USB Framework and are found
	in FW.c   	
=============================================================================*/

extern BOOL GotSUD;             // Received setup data flag
extern BOOL Sleep;			   	// see TRM Chapter 6 Power Management
extern BOOL Rwuen;				// see TRM Chapter 6 Power Management
extern BOOL Selfpwr;			// see TRM Chapter 6 Power Management

BYTE Configuration;             // Current configuration
BYTE AlternateSetting;          // Alternate settings


// FX2 USB PortA = slave fifo enable(s) when IFCFG[1:0] = 11
//sbit PA0 = IOA ^ 0; // alt. function INT 0
//sbit PA1 = IOA ^ 1; // alt. function INT 1
//sbit PA2 = IOA ^ 2; // is sloe
//sbit PA3 = IOA ^ 3; // alt. function WU2
//sbit PA4 = IOA ^ 4;	// is FIFO Adress 0
//sbit PA5 = IOA ^ 5; // is FIFO Adress 1
//sbit PA6 = IOA ^ 6; // is PKTEND
//sbit PA7 = IOA ^ 7; // is FLAGS / SLCS

//-------------------------------------DEFINE fr test ausgabe ---
#define PORTOUTCFGA  OEA 
#define PORTOUTCFGB  OEB 

//-------------------------------------------- defines
#define TIMERSTART 	T2CON |=  BIT2;
#define TIMERSTOP	T2CON &= ~BIT2; // ~ invertiert alle bits in der Zahl

//#define MINUTECOUNTER 	4000  //4000 * 15ms
//#define TEMPCOUNTER		 200  // 200 * 15ms

#define MINUTECOUNTER 	    1000  //1000 * 60ms _ eine minute
#define TEMPCOUNTER		      50  //  50 * 60ms _ 3 sekunden
// zeit zwischen zwei berlufen:
// 5545^d = 0x15A9 --> 15.00ms 
//T unsigned short T0Relaod = 0x15A9;
unsigned short T0Relaod = 0x0738; // 0x15A9/3 fr debuging, damit der timer schneller luft.


void SetupCommand(void);

unsigned short minutecnt, tempcnt;
unsigned char DoMinuteCount, DoTempGet;
unsigned long OperationCnt, Cnthelp;
/*=============================================================================
	Task Dispatcher hooks
	The following hooks are called by the task dispatcher.
=============================================================================*/
void init_fx2(void) //------------------------------- TEST AUSGABE
{ 
    CPUCS = 0x10;    // 48MHz CPU-Clock 
    EP1OUTBC = 0x01;   // Writing to EP1OUTBC rearms for an out Transfer 
    PORTOUTCFGA = 0xff;   // Ausgabeport auf Ausgabe stellen 
    PORTOUTCFGB = 0xff; 
    IOA = 0x00;   // Auf Null setzen... 

} 


void TD_Init(void)             // Called once at startup
{

   BREAKPT &= ~bmBPEN;      // to see BKPT LED go out TGE
   Rwuen = TRUE;            // Enable remote-wakeup

    init_fx2();    // Unterfunktionsaufruf 
/* initialize timer T0 */

	DoMinuteCount = 0;
	DoTempGet = 0;
	minutecnt = MINUTECOUNTER;											// set counter for minute countdown
	tempcnt = TEMPCOUNTER - 3;											// set counter for temperature aquisition

	CKCON 	= 0x00;
	TMOD	= 0x01;
	TCON	= 0x00;

	TF0 	= 0;
	EA		= 1;														// Global Interrupt Enable
	ET0		= 1;														// Timer0 Interrupt Enable

	TL0 = (T0Relaod & 0xFF);											// load Timer0 LowByte
	TH0 = ((T0Relaod>>8) &0xFF);										// load Timer0 HighByte

	TR0 	= 1;														// start Timer0

}


void TD_Poll(void)              // Called repeatedly while the device is idle
{

}

BOOL TD_Suspend(void)          // Called before the device goes into suspend mode
{
   return(TRUE);
}

BOOL TD_Resume(void)          // Called after the device resumes
{
   return(TRUE);
}

//-----------------------------------------------------------------------------
// Device Request hooks
//   The following hooks are called by the end point 0 device request parser.
//-----------------------------------------------------------------------------

BOOL DR_GetDescriptor(void)
{
   return(TRUE);
}

BOOL DR_SetConfiguration(void)   
// Called when a Set Configuration command is received
{
   if (EZUSB_HIGHSPEED())
	{
	// ... FX2 in highspeed mode
	EP6AUTOINLENH = 0x02; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x00; //	to 512 bytes
	}
	else
	{
	// ... FX2 in fullspeed mode
	EP6AUTOINLENH = 0x00; // 	configures AUTOINLEN as transmission trigger
	EP6AUTOINLENL = 0x40; //	to 64 bytes
	}
	Configuration = SETUPDAT[ 2 ];
	return(TRUE);            // Handled by user code
}

BOOL DR_GetConfiguration(void)   
// Called when a Get Configuration command is received
{
   EP0BUF[ 0 ] = Configuration;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_SetInterface(void)       
// Called when a Set Interface command is received
{
   AlternateSetting = SETUPDAT[ 2 ];
   return(TRUE);            // Handled by user code
}

BOOL DR_GetInterface(void)       
// Called when a Set Interface command is received
{
   EP0BUF[0] = AlternateSetting;
   EP0BCH = 0;
   EP0BCL = 1;
   return(TRUE);            // Handled by user code
}

BOOL DR_GetStatus(void)
{
   return(TRUE);
}

BOOL DR_ClearFeature(void)
{
   return(TRUE);
}

BOOL DR_SetFeature(void)
{
   return(TRUE);
}

BOOL DR_VendorCmnd(void)
{
  return(TRUE);
}

//-----------------------------------------------------------------------------
// USB Interrupt Handlers
//   The following functions are called by the USB interrupt jump table.
//-----------------------------------------------------------------------------


/*========================================================================================================================*/
void timer0_ISR (void) interrupt 1                                  //This function is an interrupt service routine for TIMER 0.
{																// It will be executed automatically when TIMER 0 overflows.

	int i;

	IOA |= 0x01;													// remove!
	TR0 = 0;														// stop Timer0
	TL0 = (T0Relaod & 0xFF);										// reload Timer0 LowByte
	TH0 = ((T0Relaod>>8) &0xFF);									// reload Timer0 HighByte
	TR0 = 1;														// start Timer0
	if(!(--minutecnt)) { DoMinuteCount = 1; }						// set Flag for Minute Increment
	if(!(--tempcnt)) { DoTempGet = 1; }								// set Flag for Temperature Aquisition
	IOA &= ~0x01;													// remove!
   // TEST AUSGABE HIER !!! wird berhaupt das Prog. gebootet?! ------------------


	// A0 etwas lnger angesteuert lassen.
	for(i=0;i<1000;i++)
	{
		IOA = 1;
	}
	
	for(i=0;i<1000;i++)
	{
		IOA = 0;
	}
//----------------------------------------- ENDE TEST AUSGABE-------------------

	return;
}/*========================================================================================================================*/



// Setup Data Available Interrupt Handler
void ISR_Sudav(void) interrupt 0
{
   GotSUD = TRUE;            // Set flag
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUDAV;         // Clear SUDAV IRQ
   SetupCommand();	// speed up, if it is 'here' -> see USB Spec.
}

// Setup Token Interrupt Handler
void ISR_Sutok(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUTOK;         // Clear SUTOK IRQ
}

void ISR_Sof(void) interrupt 0
{
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSOF;            // Clear SOF IRQ
}

void ISR_Ures(void) interrupt 0
{
   /* Commented defaults:
   // whenever we get a USB reset, we should revert to full speed mode
   pConfigDscr = pFullSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
   pOtherConfigDscr = pHighSpeedConfigDscr;
   ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ
   */

   if ( EZUSB_HIGHSPEED() )
   { 
   	pConfigDscr = pHighSpeedConfigDscr;
	pOtherConfigDscr = pFullSpeedConfigDscr;
   }
   else
   {
	pConfigDscr = pFullSpeedConfigDscr;
	pOtherConfigDscr = pHighSpeedConfigDscr;
   }
   
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmURES;         // Clear URES IRQ	    	
}

void ISR_Susp(void) interrupt 0
{
   Sleep = TRUE;
   EZUSB_IRQ_CLEAR();
   USBIRQ = bmSUSP;
}

void ISR_Highspeed(void) interrupt 0
{
   if (EZUSB_HIGHSPEED())
   {
      pConfigDscr = pHighSpeedConfigDscr;
      //((CONFIGDSCR xdata *) pConfigDscr)->type = CONFIG_DSCR;
      pOtherConfigDscr = pFullSpeedConfigDscr;
     // ((CONFIGDSCR xdata *) pOtherConfigDscr)->type = OTHERSPEED_DSCR;
   }
   else
   {
	pConfigDscr = pFullSpeedConfigDscr;
	pOtherConfigDscr = pHighSpeedConfigDscr;
   }

   EZUSB_IRQ_CLEAR();
   USBIRQ = bmHSGRANT;
}
void ISR_Ep0ack(void) interrupt 0
{
}
void ISR_Stub(void) interrupt 0
{
}
void ISR_Ep0in(void) interrupt 0
{
}
void ISR_Ep0out(void) interrupt 0
{
}
void ISR_Ep1in(void) interrupt 0
{
}
void ISR_Ep1out(void) interrupt 0
{
}
void ISR_Ep2inout(void) interrupt 0
{
}
void ISR_Ep4inout(void) interrupt 0
{
}
void ISR_Ep6inout(void) interrupt 0
{
}
void ISR_Ep8inout(void) interrupt 0
{
}
void ISR_Ibn(void) interrupt 0
{
}
void ISR_Ep0pingnak(void) interrupt 0
{
}
void ISR_Ep1pingnak(void) interrupt 0
{
}
void ISR_Ep2pingnak(void) interrupt 0
{
}
void ISR_Ep4pingnak(void) interrupt 0
{
}
void ISR_Ep6pingnak(void) interrupt 0
{
}
void ISR_Ep8pingnak(void) interrupt 0
{
}
void ISR_Errorlimit(void) interrupt 0
{
}
void ISR_Ep2piderror(void) interrupt 0
{
}
void ISR_Ep4piderror(void) interrupt 0
{
}
void ISR_Ep6piderror(void) interrupt 0
{
}
void ISR_Ep8piderror(void) interrupt 0
{
}
void ISR_Ep2pflag(void) interrupt 0
{
}
void ISR_Ep4pflag(void) interrupt 0
{
}
void ISR_Ep6pflag(void) interrupt 0
{
}
void ISR_Ep8pflag(void) interrupt 0
{
}
void ISR_Ep2eflag(void) interrupt 0
{
}
void ISR_Ep4eflag(void) interrupt 0
{
}
void ISR_Ep6eflag(void) interrupt 0
{
}
void ISR_Ep8eflag(void) interrupt 0
{
}
void ISR_Ep2fflag(void) interrupt 0
{
}
void ISR_Ep4fflag(void) interrupt 0
{
}
void ISR_Ep6fflag(void) interrupt 0
{
}
void ISR_Ep8fflag(void) interrupt 0
{
}
void ISR_GpifComplete(void) interrupt 0
{
}
void ISR_GpifWaveform(void) interrupt 0
{
}
/*
void ISR_int1(void) interrupt 2 using 2
{
}
*/